@@ -339,257 +339,58 @@
 //! assert_eq!(!&Answer::Yes, Answer::No);
 //! assert_eq!(!&Answer::No, Answer::Yes);
 //! ```
+//!
+//! ### Making an operation commutative
+//!
+//! There are also macros to automatically make an operation commutative. That is, for two types `T` and `U`, if `T binop U` is implemented, then one can use [`commutative_binop`] to automatically implement `U binop T`. If `T` and `U` are additionally `Copy`, then `T binop &U`, `&T binop U`, `&T binop &U`, `U binop &T`, `&U binop T` and `&U binop &T` can automatically be implemented with [`forward_ref_commutative_binop`].
+//!
+//! ```
+//! use std::ops::Add;
+//! use forward_ref_generic::{commutative_binop, forward_ref_commutative_binop};
+//!
+//! // two wrappers for integers
+//! #[derive(Clone, Copy, PartialEq)]
+//! struct Int1(i32);
+//!
+//! #[derive(Clone, Copy, PartialEq)]
+//! struct Int2(i32);
+//!
+//! impl Add<Int2> for Int1 {
+//!     type Output = i32;
+//!
+//!     fn add(self, rhs: Int2) -> Self::Output {
+//!         self.0 + rhs.0
+//!     }
+//! }
+//!
+//! // note that the order of `LHS` and `RHS` is that
+//! // of the original operation's implementation
+//! // not that of the created one
+//! commutative_binop! {
+//!     impl Add for Int1, Int2
+//! }
+//!
+//! // the order of `LHS` and `RHS` here doesn't matter
+//! // as `LHS binop RHS` and `RHS binop LHS` are both required anyway
+//! forward_ref_commutative_binop! {
+//!     impl Add for Int1, Int2
+//! }
+//!
+//! let i1 = Int1(5);
+//! let i2 = Int2(3);
+//!
+//! assert_eq!(i1 + i2, 8);
+//! assert_eq!(i2 + i1, 8);
+//!
+//! assert_eq!(&i1 + i2, 8);
+//! assert_eq!(i1 + &i2, 8);
+//! assert_eq!(&i1 + &i2, 8);
+//!
+//! assert_eq!(&i2 + i1, 8);
+//! assert_eq!(i2 + &i1, 8);
+//! assert_eq!(&i2 + &i1, 8);
+//! ```
 
-/// For a type `T: Copy` which has unary operator `unop` implemented, also implement `unop &T`.
-///
-/// For readability, the expected syntax of the macro is the following:
-/// ```text
-/// ( [ Generics ] )?
-/// impl Trait, Method for Type
-/// ( where Bounds )?
-/// ```
-/// - `Generics` are comma-seperated type or const generics
-/// - `Trait` is the trait to be implemented
-/// - `Method` is the method that `Trait` defines\
-///   (can be ommitted for [`Neg`](https://doc.rust-lang.org/std/ops/trait.Neg.html))
-/// - `Type` is the type that `Trait` is implemented on (i.e. `T`)
-/// - `Bounds` are comma-seperated trait bounds for the listed generics
-#[macro_export]
-macro_rules! forward_ref_unop {
-    (
-        $( [ $($generic:tt)* ] )?
-        impl Neg for $type:ty
-        $( where $($bound:tt)* )?
-    ) => {
-        forward_ref_unop! {
-            $( [ $($generic)* ] )?
-            impl Neg, neg for $type
-            $( where $($bound)* )?
-        }
-    };
-
-    (
-        $( [ $($generic:tt)* ] )?
-        impl $impl:ident, $meth:ident for $type:ty
-        $( where $($bound:tt)* )?
-    ) => {
-        impl$(<$($generic)*>)? $impl for &$type
-        $(where
-            $($bound)*)?
-        {
-            type Output = <$type as $impl>::Output;
-
-            fn $meth(self) -> Self::Output {
-                <$type>::$meth(*self)
-            }
-        }
-    };
-}
-
-/// For types `T: Copy`, `U: Copy` for which binary operator `binop` is implemented (`T binop U`), also implement `T binop &U`, `&T binop U` and `&T binop &U`.
-///
-/// For readability, the expected syntax of the macro is the following:
-/// ```text
-/// ( [ Generics ] )?
-/// impl Trait, Method for LHS(, RHS)?
-/// ( where Bounds )?
-/// ```
-/// - `Generics` are comma-seperated type or const generics
-/// - `Trait` is the trait to be implemented
-/// - `Method` is the method that `Trait` defines\
-///   (can be ommitted for [`Add`](https://doc.rust-lang.org/std/ops/trait.Add.html), [`Sub`](https://doc.rust-lang.org/std/ops/trait.Sub.html), [`Mul`](https://doc.rust-lang.org/std/ops/trait.Mul.html) and [`Div`](https://doc.rust-lang.org/std/ops/trait.Div.html))
-/// - `LHS` is the type of the left hand side of the operation (i.e. `T`)
-/// - `RHS` is the type of the right hand side of the operation (i.e. `U`)\
-///   if no `RHS` is given, `LHS` = `RHS` is assumed
-/// - `Bounds` are comma-seperated trait bounds for the listed generics
-#[macro_export]
-macro_rules! forward_ref_binop {
-    (
-        $( [ $($generic:tt)* ] )?
-        impl Add for $lhs:ty $(, $rhs:ty )?
-        $( where $($bound:tt)* )?
-    ) => {
-        forward_ref_binop! {
-            $( [ $($generic)* ] )?
-            impl Add, add for $lhs $(, $rhs )?
-            $( where $($bound)* )?
-        }
-    };
-    (
-        $( [ $($generic:tt)* ] )?
-        impl Sub for $lhs:ty $(, $rhs:ty )?
-        $( where $($bound:tt)* )?
-    ) => {
-        forward_ref_binop! {
-            $( [ $($generic)* ] )?
-            impl Sub, sub for $lhs $(, $rhs )?
-            $( where $($bound)* )?
-        }
-    };
-    (
-        $( [ $($generic:tt)* ] )?
-        impl Mul for $lhs:ty $(, $rhs:ty )?
-        $( where $($bound:tt)* )?
-    ) => {
-        forward_ref_binop! {
-            $( [ $($generic)* ] )?
-            impl Mul, mul for $lhs $(, $rhs )?
-            $( where $($bound)* )?
-        }
-    };
-    (
-        $( [ $($generic:tt)* ] )?
-        impl Div for $lhs:ty $(, $rhs:ty )?
-        $( where $($bound:tt)* )?
-    ) => {
-        forward_ref_binop! {
-            $( [ $($generic)* ] )?
-            impl Div, div for $lhs $(, $rhs )?
-            $( where $($bound)* )?
-        }
-    };
-
-    // if no RHS was given, assume RHS = LHS
-    (
-        $( [ $($generic:tt)* ] )?
-        impl $impl:ident, $meth:ident for $lhs:ty
-        $( where $($bound:tt)* )?
-    ) => {
-        forward_ref_binop! {
-            $( [ $($generic)* ] )?
-            impl $impl, $meth for $lhs, $lhs
-            $( where $($bound)* )?
-        }
-    };
-
-    (
-        $( [ $($generic:tt)* ] )?
-        impl $impl:ident, $meth:ident for $lhs:ty, $rhs:ty
-        $( where $($bound:tt)* )?
-    ) => {
-        impl$(<$($generic)*>)? $impl<$rhs> for &$lhs
-        $(where
-            $($bound)*)?
-        {
-            type Output = <$lhs as $impl<$rhs>>::Output;
-
-            fn $meth(self, rhs: $rhs) -> Self::Output {
-                <$lhs>::$meth(*self, rhs)
-            }
-        }
-
-        impl$(<$($generic)*>)? $impl<&$rhs> for $lhs
-        $(where
-            $($bound)*)?
-        {
-            type Output = <$lhs as $impl<$rhs>>::Output;
-
-            fn $meth(self, rhs: &$rhs) -> Self::Output {
-                <$lhs>::$meth(self, *rhs)
-            }
-        }
-
-        impl$(<$($generic)*>)? $impl<&$rhs> for &$lhs
-        $(where
-            $($bound)*)?
-        {
-            type Output = <$lhs as $impl<$rhs>>::Output;
-
-            fn $meth(self, rhs: &$rhs) -> Self::Output {
-                <$lhs>::$meth(*self, *rhs)
-            }
-        }
-    };
-}
-
-/// For types `T: Copy`, `U: Copy` for which assignment operator `assop` is implemented (`T assop U`), also implement `T assop &U`.
-///
-/// For readability, the expected syntax of the macro is the following:
-/// ```text
-/// ( [ Generics ] )?
-/// impl Trait, Method for LHS(, RHS)?
-/// ( where Bounds )?
-/// ```
-/// - `Generics` are comma-seperated type or const generics
-/// - `Trait` is the trait to be implemented
-/// - `Method` is the method that `Trait` defines\
-///   (can be ommitted for [`AddAssign`](https://doc.rust-lang.org/std/ops/trait.AddAssign.html), [`SubAssign`](https://doc.rust-lang.org/std/ops/trait.SubAssign.html), [`MulAssign`](https://doc.rust-lang.org/std/ops/trait.MulAssign.html) and [`DivAssign`](https://doc.rust-lang.org/std/ops/trait.DivAssign.html))
-/// - `LHS` is the type of the left hand side of the operation (i.e. `T`)
-/// - `RHS` is the type of the right hand side of the operation (i.e. `U`)\
-///   if no `RHS` is given, `LHS` = `RHS` is assumed
-/// - `Bounds` are comma-seperated trait bounds for the listed generics
-#[macro_export]
-macro_rules! forward_ref_op_assign {
-    (
-        $( [ $($generic:tt)* ] )?
-        impl AddAssign for $lhs:ty $(, $rhs:ty )?
-        $( where $($bound:tt)* )?
-    ) => {
-        forward_ref_op_assign! {
-            $( [ $($generic)* ] )?
-            impl AddAssign, add_assign for $lhs $(, $rhs )?
-            $( where $($bound)* )?
-        }
-    };
-    (
-        $( [ $($generic:tt)* ] )?
-        impl SubAssign for $lhs:ty $(, $rhs:ty )?
-        $( where $($bound:tt)* )?
-    ) => {
-        forward_ref_op_assign! {
-            $( [ $($generic)* ] )?
-            impl SubAssign, sub_assign for $lhs $(, $rhs )?
-            $( where $($bound)* )?
-        }
-    };
-    (
-        $( [ $($generic:tt)* ] )?
-        impl MulAssign for $lhs:ty $(, $rhs:ty )?
-        $( where $($bound:tt)* )?
-    ) => {
-        forward_ref_op_assign! {
-            $( [ $($generic)* ] )?
-            impl MulAssign, mul_assign for $lhs $(, $rhs )?
-            $( where $($bound)* )?
-        }
-    };
-    (
-        $( [ $($generic:tt)* ] )?
-        impl DivAssign for $lhs:ty $(, $rhs:ty )?
-        $( where $($bound:tt)* )?
-    ) => {
-        forward_ref_op_assign! {
-            $( [ $($generic)* ] )?
-            impl DivAssign, div_assign for $lhs $(, $rhs )?
-            $( where $($bound)* )?
-        }
-    };
-
-    // if no RHS was given, assume RHS = LHS
-    (
-        $( [ $($generic:tt)* ] )?
-        impl $impl:ident, $meth:ident for $lhs:ty
-        $( where $($bound:tt)* )?
-    ) => {
-        forward_ref_op_assign! {
-            $( [ $($generic)* ] )?
-            impl $impl, $meth for $lhs, $lhs
-            $( where $($bound)* )?
-        }
-    };
-
-    (
-        $( [ $($generic:tt)* ] )?
-        impl $impl:ident, $meth:ident for $lhs:ty, $rhs:ty
-        $( where $($bound:tt)* )?
-    ) => {
-        impl$(<$($generic)*>)? $impl<&$rhs> for $lhs
-        $(where
-            $($bound)*)?
-        {
-            fn $meth(&mut self, rhs: &$rhs) {
-                <$lhs>::$meth(self, *rhs)
-            }
-        }
-    };
-}
+mod assignment;
+mod binary;
+mod unary;
